@@ -17,7 +17,7 @@ typedef struct mnt_pairing_data_s *mnt_pairing_data_ptr;
 
 struct even_mnt_pairing_data_s {
     field_t Fq, Fqx, Fqd, Fqk;
-    curve_t Eq, Etwist;
+    curve_t Eq, Etwist, Eqk;
     element_t nqrinv, nqrinv2;
     mpz_t tateexp;
     int k;
@@ -392,7 +392,7 @@ static void pairing_init_c_param_odd_k(pairing_t pairing, c_param_t param)
     mpz_clear(z);
     p->k = param->k;
     pairing->GT = p->Fqk;
-    pairing->phi = trace;
+    //pairing->phi = trace;
 
     element_clear(a);
     element_clear(b);
@@ -880,6 +880,53 @@ static void Fq_to_Fqd_to_Fqk(element_t out, element_t in)
     element_set0(fi_im(out));
 }
 
+static void even_k_trace(element_t out, element_t in, pairing_ptr pairing)
+{
+    int i;
+    point_ptr p = in->data;
+    point_t r;
+    point_t q;
+    point_ptr outp = out->data;
+    even_mnt_pairing_data_ptr pdp = pairing->data;
+
+    point_init(q, pdp->Eqk);
+    point_init(r, pdp->Eqk);
+
+    q->inf_flag = 0;
+    //map from twist: (x, y) --> (v^-1 x, v^-(3/2) y)
+    //where v is the quadratic nonresidue used to construct the twist
+    element_mul(fi_re(q->x), p->x, pdp->nqrinv);
+    //v^-3/2 = v^-2 * v^1/2
+    element_mul(fi_im(q->y), p->y, pdp->nqrinv2);
+    point_set(r, q);
+
+    printf("r is ");
+    point_out_str(stdout, 0, r);
+    printf("\n");
+
+    for (i=1; i<pdp->k; i++) {
+	cc_frobenius(q, q, pdp->Fq->order);
+    printf("q is ");
+    point_out_str(stdout, 0, q);
+    printf("\n");
+
+	point_add(r, r, q);
+    printf("r is ");
+    point_out_str(stdout, 0, r);
+    printf("\n");
+    }
+    point_clear(q);
+
+    printf("r is \n");
+    point_out_str(stdout, 0, r);
+    printf("\n");
+    exit(1);
+    element_set(outp->x, polymod_coeff(fi_re(r->x), 0));
+    element_set(outp->y, polymod_coeff(fi_re(r->y), 0));
+    //may have to multiply to ensure order is correct
+    point_clear(r);
+}
+
 static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
 {
     even_mnt_pairing_data_ptr p;
@@ -948,6 +995,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
 
     p->mapbase = Fq_to_Fqd_to_Fqk;
 
+    cc_init_map_curve(p->Eqk, p->Eq, p->Fqk, p->mapbase);
     cc_init_map_curve(p->Etwist, p->Eq, p->Fqd, element_field_to_polymod);
     twist_curve(p->Etwist);
     element_init(p->nqrinv, p->Fqd);
@@ -965,7 +1013,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
     mpz_clear(one);
     p->k = param->k;
     pairing->GT = p->Fqk;
-    //pairing->phi = trace;
+    pairing->phi = even_k_trace;
 
     element_clear(a);
     element_clear(b);
@@ -1104,8 +1152,6 @@ void c_param_from_cm(c_param_t param, cm_info_ptr cm)
     field_init_polymod(Fqd, irred);
 
     //find a quadratic nonresidue of Fqd lying in Fq
-    //TODO: we're assuming there exists an element in Fq
-    //that is not a square in Fqd
     element_init(nqr, Fqd);
     do {
 	element_random(((element_t *) nqr->data)[0]);
