@@ -245,11 +245,22 @@ void element_pow3(element_ptr x, element_ptr a1, mpz_ptr n1,
 element_ptr field_get_nqr(field_ptr f)
 {
     if (!f->nqr) {
+	//use a deterministic random number generator
+	//so that the quadratic nonresidue is the same every time
+	gmp_randstate_t rs;
+	void internal_random(mpz_t z, mpz_t limit, void *data)
+	{
+	    mpz_urandomm(z, rs, limit);
+	}
+	gmp_randinit_default(rs);
+	random_push(internal_random, NULL);
+
 	f->nqr = malloc(sizeof(element_t));
 	element_init(f->nqr, f);
 	do {
 	    element_random(f->nqr);
 	} while (element_is_sqr(f->nqr));
+	random_pop();
     }
     return f->nqr;
 }
