@@ -71,6 +71,12 @@ static void fq_out_str(FILE *stream, int base, element_ptr e)
     fputc(']', stream);
 }
 
+static int fq_sign(element_ptr n)
+{
+    fq_data_ptr r = n->data;
+    return element_sign(r->x);
+}
+
 static void fq_add(element_ptr n, element_ptr a, element_ptr b)
 {
     fq_data_ptr p = a->data;
@@ -320,6 +326,7 @@ void field_init_quadratic(field_ptr f, field_ptr fbase)
     f->set_si = fq_set_si;
     f->set_mpz = fq_set_mpz;
     f->out_str = fq_out_str;
+    f->sign = fq_sign;
     f->add = fq_add;
     f->sub = fq_sub;
     f->set = fq_set;
@@ -497,6 +504,7 @@ void field_init_fi(field_ptr f, field_ptr fbase)
     f->set_si = fq_set_si;
     f->set_mpz = fq_set_mpz;
     f->out_str = fq_out_str;
+    f->sign = fq_sign;
     f->add = fq_add;
     f->sub = fq_sub;
     f->set = fq_set;
