@@ -2,7 +2,7 @@
 #include <sys/time.h>
 #include <time.h>
 
-double get_time()
+double get_time(void)
 {
     static struct timeval last_tv, tv;
     static int first = 1;
