@@ -5,15 +5,11 @@ void bls_sign(unsigned char *sig, int hashlen, void *hash,
 	bls_private_key_t sk)
 {
     element_t h;
-    point_ptr P;
 
     element_init(h, sk->param->pairing->G1);
     element_from_hash(h, hashlen, hash);
     element_pow(h, h, sk->x);
-    //unfortunately we must give up the element_t abstraction
-    //since we only want to output the x coordinate
-    P = h->data;
-    element_to_bytes(sig, P->x);
+    element_to_bytes_x_only(sig, h);
 
     element_clear(h);
 }
@@ -22,23 +18,18 @@ int bls_verify(unsigned char *sig, int hashlen, void *hash,
 	bls_public_key_t pk)
 {
     //have to mess with internals since we are only given the x-coord
-    element_t x;
     element_t hx;
     element_t h;
     int res;
 
     field_ptr G1 = pk->param->pairing->G1;
-    curve_group_data_ptr p = G1->data;
 
     element_init(h, G1);
     element_from_hash(h, hashlen, hash);
     element_init(hx, G1);
-    element_init(x, p->curve->field);
-    element_from_bytes(x, sig);
-    point_from_x((point_ptr) hx->data, x);
+    element_from_bytes_x_only(hx, sig);
 
     res = is_almost_coddh(h, hx, pk->param->g, pk->gx, pk->param->pairing);
-    element_clear(x);
     element_clear(hx);
     return res;
 }
